@@ -28,7 +28,7 @@
 #
 #========================================================================
 #
-# Version 4.1.1, released 29 Mar 2017.
+# Version 4.2.1, released 7 May 2018.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -580,17 +580,15 @@ sub getFTPArgs
     my $conf = $t->{conf};
 
     return {
-        Host         => $conf->{ClientNameAlias}
-                     || $t->{hostIP}
-                     || $t->{host},
-#        Firewall     => undef,                            # not used
-#        FirewallType => undef,                            # not used
-#        BlockSize    => $conf->{FtpBlockSize} || 10240,
-#        Port         => $conf->{FtpPort}      || 21,
-#        Timeout      => defined($conf->{FtpTimeout}) ? $conf->{FtpTimeout} : 120,
+        Host         => $t->{hostIP} || $t->{host},
+        Firewall     => undef,                            # not used
+        FirewallType => undef,                            # not used
+        BlockSize    => $conf->{FtpBlockSize} || 10240,
+        Port         => $conf->{FtpPort}      || 21,
+        Timeout      => defined($conf->{FtpTimeout}) ? $conf->{FtpTimeout} : 120,
         Debug        => $t->{logLevel} >= 5 ? 1 : 0,
         Passive      => (defined($conf->{FtpPassive}) ? $conf->{FtpPassive} : 1),
-#        Hash         => undef,                            # do not touch
+        Hash         => undef,                            # do not touch
     };
 }
 
@@ -904,6 +902,7 @@ sub handleDir
     # If we didn't see a file, move to old.
     #
     foreach my $name ( keys(%$all) ) {
+        next if ( $name eq "." || $name eq ".." );
         my $path = "$f->{name}/$name";
         $path =~ s{^/+}{};
         $t->logWrite("dirCleanup: checking $path, expected = $expectedFiles{$path}\n", 5);
@@ -996,7 +995,7 @@ sub handleFile
     if ( !*FTP || $@ || $errs ) {
         $t->logFileAction( "fail", $f->{name}, $f );
         $t->{xferBadFileCnt}++;
-        $stats->{errCnt} += scalar @$errs;
+        $stats->{errCnt} += ref($errs) eq 'ARRAY' ? scalar(@$errs) : 1;
         return;
     }
 
@@ -1064,7 +1063,7 @@ sub logFileAction
         $owner, $attrib->{size}, $attrib->{name}
     );
 
-    if ( ($t->{stats}{TotalFileCnt} % 20) == 0 ) {
+    if ( ($t->{stats}{TotalFileCnt} % 20) == 0 && !$t->{noProgressPrint} ) {
         printf("__bpc_progress_fileCnt__ %d\n", $t->{stats}{TotalFileCnt});
     }
 
@@ -1094,60 +1093,46 @@ sub moveFileToOld
         return;
     }
     $t->logWrite("moveFileToOld $a->{name}, $f->{name}, links = $a->{nlinks}, type = $a->{type}\n", 5);
-    if ( !$AttrOld || $AttrOld->get($f->{name}) ) {
-        if ( $a->{nlinks} > 0 ) {
-            $a->{nlinks}--;
-            if ( $a->{nlinks} <= 0 ) {
-                $AttrNew->deleteInode($a->{inode});
-                $DeltaNew->update($a->{compress}, $a->{digest}, -1);
-            } else {
-                $AttrNew->setInode($a->{inode}, $a);
-            }
+    if ( $a->{nlinks} > 0 ) {
+        $a->{nlinks}--;
+        if ( $a->{nlinks} <= 0 ) {
+            $AttrNew->deleteInode($a->{inode});
+            $DeltaNew->update($a->{compress}, $a->{digest}, -1);
         } else {
-            $DeltaNew->update($a->{compress}, $a->{digest}, -1)
-                                            if ( length($a->{digest}) );
+            $AttrNew->setInode($a->{inode}, $a);
         }
-        $AttrNew->delete($f->{name});
-        if ( $a->{type} == BPC_FTYPE_DIR ) {
+    } else {
+        $DeltaNew->update($a->{compress}, $a->{digest}, -1)
+                                        if ( length($a->{digest}) );
+        if ( $AttrOld && $AttrOld->get($f->{name}) && $AttrOld->set($f->{name}, $a, 1) ) {
+            $DeltaOld->update($a->{compress}, $a->{digest}, 1);
+        }
+    }
+    $AttrNew->delete($f->{name});
+    if ( $a->{type} == BPC_FTYPE_DIR ) {
+        if ( !$AttrOld || $AttrOld->get($f->{name}) ) {
             #
             # Delete the directory tree, including updating reference counts
             #
             my $pathNew = $AttrNew->getFullMangledPath($f->{name});
             BackupPC::DirOps::RmTreeQuiet($bpc, $pathNew, $a->{compress}, $DeltaNew, $AttrNew);
-        }
-        return;
-    }
-
-    if ( $a->{nlinks} > 0 ) {
-        #
-        # only write the inode if it doesn't exist in old;
-        # in that case, increase the pool reference count
-        #
-        if ( $AttrOld->set($f->{name}, $a, 1) ) {
-            $DeltaOld->update($a->{compress}, $a->{digest}, 1);
-        }
-    } else {
-        $AttrOld->set($f->{name}, $a);
-        $DeltaOld->update($a->{compress}, $a->{digest}, 1);
-    }
-    $AttrNew->delete($f->{name});
-    $DeltaNew->update($a->{compress}, $a->{digest}, -1);
-    if ( $a->{type} == BPC_FTYPE_DIR && $AttrOld ) {
-        #
-        # For a directory we need to move it to old, and copy
-        # any inodes that are referenced below this directory.
-        #
-        my $pathNew = $AttrNew->getFullMangledPath($f->{name});
-        my $pathOld = $AttrOld->getFullMangledPath($f->{name});
-        $t->logWrite("moveFileToOld(..., $f->{name}): renaming $pathNew to $pathOld\n", 5);
-        $AttrNew->flush(0, $f->{name});
-        BackupPC::XS::DirOps::refCountAll($pathNew, $a->{compress}, -1, $DeltaNew);
-        BackupPC::XS::DirOps::refCountAll($pathNew, $a->{compress},  1, $DeltaOld);
-        $t->copyInodes($f->{name});
-        $t->pathCreate($pathOld);
-        if ( !rename($pathNew, $pathOld) ) {
-            $t->logWrite("moveFileToOld(..., $f->{name}): can't rename $pathNew to $pathOld\n", 1);
-            $t->{xferErrCnt}++;
+        } else {
+            #
+            # For a directory we need to move it to old, and copy
+            # any inodes that are referenced below this directory.
+            #
+            my $pathNew = $AttrNew->getFullMangledPath($f->{name});
+            my $pathOld = $AttrOld->getFullMangledPath($f->{name});
+            $t->logWrite("moveFileToOld(..., $f->{name}): renaming $pathNew to $pathOld\n", 5);
+            $AttrNew->flush(0, $f->{name});
+            BackupPC::XS::DirOps::refCountAll($pathNew, $a->{compress}, -1, $DeltaNew);
+            BackupPC::XS::DirOps::refCountAll($pathNew, $a->{compress},  1, $DeltaOld);
+            $t->copyInodes($f->{name});
+            $t->pathCreate($pathOld);
+            if ( !rename($pathNew, $pathOld) ) {
+                $t->logWrite("moveFileToOld(..., $f->{name}): can't rename $pathNew to $pathOld\n", 1);
+                $t->{xferErrCnt}++;
+            }
         }
     }
 }
@@ -1168,7 +1153,6 @@ sub copyInodes
     $t->logWrite("copyInodes: dirName = $dirName, dirPath = $dirPath\n", 4);
 
     my $attrAll = $AttrNew->getAll($dirName);
-    $t->logWrite("copyInodes: finished getAll()\n", 4);
     $bpc->flushXSLibMesgs();
 
     #
@@ -1192,6 +1176,7 @@ sub copyInodes
     }
 
     foreach my $fileUM ( keys(%$attrAll) ) {
+        next if ( $fileUM eq "." || $fileUM eq ".." );
         my $a = $attrAll->{$fileUM};
         if ( $a->{type} == BPC_FTYPE_DIR ) {
             #
